@@ -16,6 +16,8 @@
 	IBOutlet NSView* gameSettingsView;
 	IBOutlet NSView* fontSettingsView;
 	IBOutlet NSView* colourSettingsView;
+	IBOutlet NSView* typographicalSettingsView;
+	IBOutlet NSView* displaySettingsView;
 	
 	// The settings controls themselves
 	IBOutlet NSButton* displayWarnings;
@@ -45,6 +47,18 @@
 	IBOutlet NSTableView* fonts;
 	IBOutlet NSTableView* colours;
 	
+	IBOutlet NSButton* showMargins;
+	IBOutlet NSSlider* marginWidth;
+	IBOutlet NSButton* useScreenFonts;
+	IBOutlet NSButton* useHyphenation;
+	IBOutlet NSButton* kerning;
+	IBOutlet NSButton* ligatures;
+	
+	IBOutlet NSPopUpButton* foregroundColour;
+	IBOutlet NSPopUpButton* backgroundColour;
+	IBOutlet NSButton* zoomBorders;
+	IBOutlet NSButton* showCoverPicture;
+	
 	// The toolbar
 	NSToolbar* toolbar;
 	
@@ -72,4 +86,14 @@
 - (IBAction) resetOrganiseDir: (id) sender;
 - (IBAction) reorganiseGames: (id) sender;
 
+- (IBAction) marginsChanged: (id) sender;
+- (IBAction) screenFontsChanged: (id) sender;
+- (IBAction) hyphenationChanged: (id) sender;
+- (IBAction) ligaturesChanged: (id) sender;
+- (IBAction) kerningChanged: (id) sender;
+
+- (IBAction) bordersChanged: (id) sender;
+- (IBAction) showCoverPictureChanged: (id) sender;
+- (IBAction) colourChanged: (id) sender;
+
 @end
