@@ -14,8 +14,10 @@
 
 static NSToolbarItem* generalSettingsItem;
 static NSToolbarItem* gameSettingsItem;
+static NSToolbarItem* displaySettingsItem;
 static NSToolbarItem* fontSettingsItem;
 static NSToolbarItem* colourSettingsItem;
+static NSToolbarItem* typographicSettingsItem;
 
 static NSDictionary*  itemDictionary = nil;
 
@@ -25,15 +27,19 @@ static NSDictionary*  itemDictionary = nil;
 	// Create the toolbar items
 	generalSettingsItem = [[NSToolbarItem alloc] initWithItemIdentifier: @"generalSettings"];
 	gameSettingsItem = [[NSToolbarItem alloc] initWithItemIdentifier: @"gameSettings"];
+	displaySettingsItem = [[NSToolbarItem alloc] initWithItemIdentifier: @"displaySettings"];
 	fontSettingsItem = [[NSToolbarItem alloc] initWithItemIdentifier: @"fontSettings"];
 	colourSettingsItem = [[NSToolbarItem alloc] initWithItemIdentifier: @"colourSettings"];
+	typographicSettingsItem = [[NSToolbarItem alloc] initWithItemIdentifier: @"typographicSettings"];
 	
 	// ... and the dictionary
 	itemDictionary = [[NSDictionary dictionaryWithObjectsAndKeys:
 		generalSettingsItem, @"generalSettings",
 		gameSettingsItem, @"gameSettings",
+		displaySettingsItem, @"displaySettings",
 		fontSettingsItem, @"fontSettings",
 		colourSettingsItem, @"colourSettings",
+		typographicSettingsItem, @"typographicSettings",
 		nil] retain];
 	
 	// Set up the items
@@ -41,16 +47,22 @@ static NSDictionary*  itemDictionary = nil;
 	[generalSettingsItem setImage: [[[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForImageResource: @"generalSettings"]] autorelease]];
 	[gameSettingsItem setLabel: @"Game"];
 	[gameSettingsItem setImage: [[[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForImageResource: @"gameSettings"]] autorelease]];
+	[displaySettingsItem setLabel: @"Display"];
+	[displaySettingsItem setImage: [[[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForImageResource: @"displaySettings"]] autorelease]];
 	[fontSettingsItem setLabel: @"Fonts"];
 	[fontSettingsItem setImage: [[[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForImageResource: @"fontSettings"]] autorelease]];
 	[colourSettingsItem setLabel: @"Colour"];
 	[colourSettingsItem setImage: [[[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForImageResource: @"colourSettings"]] autorelease]];
+	[typographicSettingsItem setLabel: @"Typography"];
+	[typographicSettingsItem setImage: [[[NSImage alloc] initWithContentsOfFile: [[NSBundle mainBundle] pathForImageResource: @"typographicSettings"]] autorelease]];
 	
 	// And the actions
 	[generalSettingsItem setAction: @selector(generalSettings:)];
 	[gameSettingsItem setAction: @selector(gameSettings:)];
+	[displaySettingsItem setAction: @selector(displaySettings:)];
 	[fontSettingsItem setAction: @selector(fontSettings:)];
 	[colourSettingsItem setAction: @selector(colourSettings:)];	
+	[typographicSettingsItem setAction: @selector(typographicSettings:)];	
 }
 
 - (id) init {
@@ -116,7 +128,7 @@ static int familyComparer(id a, id b, void* context) {
 
 - (void) windowDidLoad {
 	// Set the toolbar
-	toolbar = [[NSToolbar allocWithZone: [self zone]] initWithIdentifier: @"preferencesToolbar"];
+	toolbar = [[NSToolbar allocWithZone: [self zone]] initWithIdentifier: @"preferencesToolbar2"];
 		
 	[toolbar setDelegate: self];
 	[toolbar setDisplayMode: NSToolbarDisplayModeIconAndLabel];
@@ -126,6 +138,11 @@ static int familyComparer(id a, id b, void* context) {
 	
 	[[self window] setContentSize: [generalSettingsView frame].size];
 	[[self window] setContentView: generalSettingsView];
+
+	if ([toolbar respondsToSelector: @selector(setSelectedItemIdentifier:)]) {
+		[toolbar setSelectedItemIdentifier: @"generalSettings"];
+	}
+	
 	
 	[fonts setDataSource: self];
 	[fonts setDelegate: self];
@@ -133,9 +150,13 @@ static int familyComparer(id a, id b, void* context) {
 	[colours setDelegate: self];
 	
 	// Set up the various font menus
-	[proportionalFont setMenu: [self fontMenu: NO]];
-	[fixedFont setMenu: [self fontMenu: YES]];
-	[symbolicFont setMenu: [self fontMenu: NO]];
+	NSMenu* proportionalMenu = [self fontMenu: NO];
+	NSMenu* fixedMenu = [self fontMenu: YES];
+	NSMenu* symbolMenu = [[proportionalMenu copy] autorelease];
+	
+	[proportionalFont setMenu: proportionalMenu];
+	[fixedFont setMenu: fixedMenu];
+	[symbolicFont setMenu: symbolMenu];
 	
 	[[NSNotificationCenter defaultCenter] addObserver: self
 											 selector: @selector(storyProgressChanged:)
@@ -148,6 +169,30 @@ static int familyComparer(id a, id b, void* context) {
 - (void) switchToPane: (NSView*) preferencePane {
 	if ([[self window] contentView] == preferencePane) return;
 	
+	// Select the appropriate item in the toolbar
+	if ([toolbar respondsToSelector: @selector(setSelectedItemIdentifier:)]) {
+		NSString* selected = nil;
+		
+		if (preferencePane == generalSettingsView) {
+			selected = @"generalSettings";
+		} else if (preferencePane == gameSettingsView) {
+			selected = @"gameSettings";
+		} else if (preferencePane == displaySettingsView) {
+			selected = @"displaySettings";
+		} else if (preferencePane == fontSettingsView) {
+			selected = @"fontSettings";
+		} else if (preferencePane == colourSettingsView) {
+			selected = @"colourSettings";
+		} else if (preferencePane == typographicalSettingsView) {
+			selected = @"typographicSettings";
+		}
+		
+		if (selected != nil) {
+			[toolbar setSelectedItemIdentifier: selected];
+		}
+	}
+	
+	// Work out the various frame sizes
 	NSRect currentFrame = [[[self window] contentView] frame];
 	NSRect oldFrame = currentFrame;
 	NSRect windowFrame = [[self window] frame];
@@ -166,6 +211,7 @@ static int familyComparer(id a, id b, void* context) {
 					display: YES
 					animate: YES];
 	[[self window] setContentView: preferencePane];
+	[[self window] setInitialFirstResponder: preferencePane];
 }
 
 // == Toolbar delegate functions ==
@@ -178,16 +224,22 @@ static int familyComparer(id a, id b, void* context) {
 
 - (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar {
     return [NSArray arrayWithObjects:
-		@"generalSettings", @"gameSettings", @"fontSettings", @"colourSettings",
+		@"generalSettings", @"gameSettings", @"displaySettings", @"fontSettings", @"typographicSettings", @"colourSettings", NSToolbarFlexibleSpaceItemIdentifier,
 		nil];
 }
 
 - (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar {
     return [NSArray arrayWithObjects:
-		NSToolbarFlexibleSpaceItemIdentifier, @"generalSettings", @"gameSettings", @"fontSettings", @"colourSettings", NSToolbarFlexibleSpaceItemIdentifier,
+		NSToolbarFlexibleSpaceItemIdentifier, @"generalSettings", @"gameSettings", @"displaySettings", @"fontSettings", @"typographicSettings", @"colourSettings", NSToolbarFlexibleSpaceItemIdentifier,
 		nil];
 }
 
+- (NSArray *)toolbarSelectableItemIdentifiers:(NSToolbar *)toolbar {
+    return [NSArray arrayWithObjects:
+		@"generalSettings", @"gameSettings", @"displaySettings", @"fontSettings", @"colourSettings", @"typographicSettings",
+		nil];	
+}
+
 // == Toolbar actions ==
 
 - (void) generalSettings: (id) sender {
@@ -198,6 +250,10 @@ static int familyComparer(id a, id b, void* context) {
 	[self switchToPane: gameSettingsView];
 }
 
+- (void) displaySettings: (id) sender {
+	[self switchToPane: displaySettingsView];
+}
+
 - (void) fontSettings: (id) sender {
 	[self switchToPane: fontSettingsView];
 }
@@ -206,6 +262,10 @@ static int familyComparer(id a, id b, void* context) {
 	[self switchToPane: colourSettingsView];
 }
 
+- (void) typographicSettings: (id) sender {
+	[self switchToPane: typographicalSettingsView];
+}
+
 // == Setting the preferences that we're editing ==
 
 - (void) setButton: (NSPopUpButton*) button
@@ -246,6 +306,73 @@ static int familyComparer(id a, id b, void* context) {
 	[fontPreview setFont: [[prefs fonts] objectAtIndex: 0]];
 }
 
+- (NSString*) colourNameAtIndex: (int) index {
+	switch (index) {
+		case 0: return @"Black";
+		case 1: return @"Red";
+		case 2: return @"Green";
+		case 3: return @"Yellow";
+		case 4: return @"Blue";
+		case 5: return @"Magenta";
+		case 6: return @"Cyan";
+		case 7: return @"White";
+		case 8: return @"Light grey";
+		case 9: return @"Medium grey";
+		case 10: return @"Dark grey";
+		default: return @"Unused colour";
+	}	
+}
+
+- (void) selectItemWithTag: (int) tag
+				   inPopup: (NSPopUpButton*) button {
+	int index;
+	
+	for (index=0; index < [[button itemArray] count]; index++) {
+		if ([[button itemAtIndex: index] tag] == tag) {
+			[button selectItemAtIndex: index];
+			return;
+		}
+	}
+}
+
+- (void) updateColourMenus {
+	NSMenu* newColourMenu = [[[NSMenu alloc] init] autorelease];
+	
+	int col;
+	for (col=0; col<10; col++) {
+		// Build the image showing a preview of this colour
+		NSImage* sampleImage = [[NSImage alloc] initWithSize: NSMakeSize(16, 12)];
+		
+		[sampleImage lockFocus];
+		[[[prefs colours] objectAtIndex: col] set];
+		NSRectFill(NSMakeRect(0,0,16,12));
+		[sampleImage unlockFocus];
+		
+		// Build the actual menu item
+		NSMenuItem* colourItem = [[NSMenuItem alloc] initWithTitle: [self colourNameAtIndex: col]
+															action: nil
+													 keyEquivalent: @""];
+		[colourItem setTag: col];
+		[colourItem setImage: sampleImage];
+		
+		// Add it to the menu
+		[newColourMenu addItem: colourItem];
+		
+		// Release our resources
+		[colourItem autorelease];
+		[sampleImage autorelease];
+	}
+	
+	// Set the menu as the menu for both the popup buttons
+	[foregroundColour setMenu: newColourMenu];
+	[backgroundColour setMenu: [[newColourMenu copy] autorelease]];
+
+	[self selectItemWithTag: [prefs foregroundColour]
+					inPopup: foregroundColour];
+	[self selectItemWithTag: [prefs backgroundColour]
+					inPopup: backgroundColour];
+}
+
 - (void) setPreferences: (ZoomPreferences*) preferences {
 	if (prefs) [prefs release];
 	prefs = [preferences retain];
@@ -271,6 +398,21 @@ static int familyComparer(id a, id b, void* context) {
 	[self setSimpleFonts];
 	
 	[organiseDir setString: [prefs organiserDirectory]];
+	
+	[showMargins setState: [prefs textMargin] > 0?NSOnState:NSOffState];
+	[useScreenFonts setState: [prefs useScreenFonts]?NSOnState:NSOffState];
+	[useHyphenation setState: [prefs useHyphenation]?NSOnState:NSOffState];
+	[kerning setState: [prefs useKerning]?NSOnState:NSOffState];
+	[ligatures setState: [prefs useLigatures]?NSOnState:NSOffState];
+	
+	[marginWidth setEnabled: [prefs textMargin] > 0];
+	if ([prefs textMargin] > 0) {
+		[marginWidth setFloatValue: [prefs textMargin]];
+	}
+	
+	[zoomBorders setState: [prefs showBorders]?NSOnState:NSOffState];
+	[showCoverPicture setState: [prefs showCoverPicture]?NSOnState:NSOffState];
+	[self updateColourMenus];
 }
 
 // == Table data source ==
@@ -333,21 +475,7 @@ static void appendStyle(NSMutableString* styleName,
 	
 	if (aTableView == colours) {
 		if ([[aTableColumn identifier] isEqualToString: @"Colour name"]) {
-			switch (rowIndex) {
-				case 0: return @"Black";
-				case 1: return @"Red";
-				case 2: return @"Green";
-				case 3: return @"Yellow";
-				case 4: return @"Blue";
-				case 5: return @"Magenta";
-				case 6: return @"Cyan";
-				case 7: return @"White";
-				case 8: return @"Light grey";
-				case 9: return @"Medium grey";
-				case 10: return @"Dark grey";
-				default: return @"Unused colour";
-			}
-			
+			return [self colourNameAtIndex: rowIndex];
 		} else if ([[aTableColumn identifier] isEqualToString: @"Colour"]) {
 			NSColor* theColour = [[prefs colours] objectAtIndex: rowIndex];
 			NSAttributedString* res;
@@ -446,13 +574,13 @@ static void appendStyle(NSMutableString* styleName,
 		[prefs setColours: cols];
 		
 		[colours reloadData];
+		[self updateColourMenus];
 	}
 	
 	[cols release];
 }
 
 - (void)changeTransparency:(id)sender {
-
 	NSMutableArray* cols = [[prefs colours] mutableCopy];
 	
 	int i;
@@ -471,7 +599,6 @@ static void appendStyle(NSMutableString* styleName,
 	[colours reloadData];
 	
 	[cols release];
-
 }
 
 // == Various actions ==
@@ -575,6 +702,61 @@ static void appendStyle(NSMutableString* styleName,
 	[self setSimpleFonts];
 }
 
+// = Typographical changes =
+
+- (IBAction) marginsChanged: (id) sender {
+	// Work out the new margin size
+	float oldSize = [prefs textMargin];
+	float newSize;
+	
+	if ([showMargins state] == NSOffState) {
+		newSize = 0;
+		[marginWidth setEnabled: NO];
+	} else if ([showMargins state] == NSOnState && oldSize <= 0) {
+		newSize = 10.0;
+		[marginWidth setEnabled: YES];
+	} else {
+		newSize = floorf([marginWidth floatValue]);
+		[marginWidth setEnabled: YES];
+	}
+	
+	if (newSize != oldSize) {
+		[prefs setTextMargin: newSize];
+	}
+}
+
+- (IBAction) screenFontsChanged: (id) sender {
+	BOOL newState = [useScreenFonts state]==NSOnState;
+	
+	if (newState != [prefs useScreenFonts]) {
+		[prefs setUseScreenFonts: newState];
+	}	
+}
+
+- (IBAction) hyphenationChanged: (id) sender {
+	BOOL newState = [useHyphenation state]==NSOnState;
+	
+	if (newState != [prefs useHyphenation]) {
+		[prefs setUseHyphenation: newState];
+	}
+}
+
+- (IBAction) ligaturesChanged: (id) sender {
+	BOOL newState = [ligatures state]==NSOnState;
+	
+	if (newState != [prefs useLigatures]) {
+		[prefs setUseLigatures: newState];
+	}	
+}
+
+- (IBAction) kerningChanged: (id) sender {
+	BOOL newState = [kerning state]==NSOnState;
+	
+	if (newState != [prefs useKerning]) {
+		[prefs setUseKerning: newState];
+	}	
+}
+
 // = Story progress meter =
 
 - (void) storyProgressChanged: (NSNotification*) not {
@@ -603,4 +785,38 @@ static void appendStyle(NSMutableString* styleName,
 	[[ZoomStoryOrganiser sharedStoryOrganiser] organiseAllStories];
 }
 
+// = Display pane =
+
+- (IBAction) bordersChanged: (id) sender {
+	BOOL newState = [sender state] == NSOnState;
+	BOOL oldState = [prefs showBorders];
+	
+	if (newState != oldState) {
+		[prefs setShowBorders: newState];
+		[prefs setShowGlkBorders: newState];
+	}
+}
+
+- (IBAction) showCoverPictureChanged: (id) sender {
+	BOOL newState = [sender state] == NSOnState;
+	BOOL oldState = [prefs showCoverPicture];
+	
+	if (newState != oldState) {
+		[prefs setShowCoverPicture: newState];
+	}	
+}
+
+- (IBAction) colourChanged: (id) sender {
+	int newValue = [sender selectedTag];
+	int oldValue = (sender==foregroundColour)?[prefs foregroundColour]:[prefs backgroundColour];
+	
+	if (newValue != oldValue) {
+		if (sender == foregroundColour) {
+			[prefs setForegroundColour: newValue];
+		} else {
+			[prefs setBackgroundColour: newValue];
+		}
+	}
+}
+
 @end
