@@ -26,16 +26,19 @@ extern NSString* ZoomPreferencesHaveChangedNotification;
 // Getting preferences
 + (NSString*) defaultOrganiserDirectory;
 
+// Warnings and game text prefs
 - (BOOL)  displayWarnings;
 - (BOOL)  fatalWarnings;
 - (BOOL)  speakGameText;
 - (BOOL)  confirmGameClose;
 - (float) scrollbackLength;	// 0-100
 
+// Interpreter preferences
 - (NSString*)     gameTitle;
 - (int)           interpreter;
 - (unsigned char) revision;
 
+// Typographical preferences
 - (NSArray*)      fonts;   // 16 fonts
 - (NSArray*)      colours; // 13 colours
 
@@ -44,12 +47,28 @@ extern NSString* ZoomPreferencesHaveChangedNotification;
 - (NSString*) symbolicFontFamily;
 - (float) fontSize;
 
-- (NSDictionary*) dictionary;
+- (float) textMargin;
+- (BOOL) useScreenFonts;
+- (BOOL) useHyphenation;
 
+- (BOOL) useKerning;
+- (BOOL) useLigatures;
+
+// Organiser preferences
 - (NSString*) organiserDirectory;
 - (BOOL)	  keepGamesOrganised;
 - (BOOL)      autosaveGames;
 
+// Display preferences
+- (int) foregroundColour;
+- (int) backgroundColour;
+- (BOOL) showBorders;
+- (BOOL) showGlkBorders;
+- (BOOL) showCoverPicture;
+
+// The dictionary
+- (NSDictionary*) dictionary;
+
 // Setting preferences
 - (void) setDisplayWarnings: (BOOL) flag;
 - (void) setFatalWarnings: (BOOL) flag;
@@ -69,10 +88,22 @@ extern NSString* ZoomPreferencesHaveChangedNotification;
 - (void) setSymbolicFontFamily: (NSString*) fontFamily;
 - (void) setFontSize: (float) size;
 
+- (void) setTextMargin: (float) textMargin;
+- (void) setUseScreenFonts: (BOOL) useScreenFonts;
+- (void) setUseHyphenation: (BOOL) useHyphenation;
+- (void) setUseKerning: (BOOL) useKerning;
+- (void) setUseLigatures: (BOOL) useLigatures;
+
 - (void) setOrganiserDirectory: (NSString*) directory;
 - (void) setKeepGamesOrganised: (BOOL) value;
 - (void) setAutosaveGames: (BOOL) value;
 
+- (void) setShowBorders: (BOOL) value;
+- (void) setShowGlkBorders: (BOOL) value;
+- (void) setForegroundColour: (int) value;
+- (void) setBackgroundColour: (int) value;
+- (void) setShowCoverPicture: (BOOL) value;
+
 // Notifications
 - (void) preferencesHaveChanged;
 
