@@ -29,9 +29,20 @@ static NSString* revision			= @"Revision";
 
 static NSString* fonts				= @"Fonts";
 static NSString* colours			= @"Colours";
+static NSString* textMargin			= @"TextMargin";
+static NSString* useScreenFonts		= @"UseScreenFonts";
+static NSString* useHyphenation		= @"UseHyphenation";
+static NSString* useKerning			= @"UseKerning";
+static NSString* useLigatures		= @"UseLigatures";
 
 static NSString* organiserDirectory = @"organiserDirectory";
 
+static NSString* foregroundColour   = @"ForegroundColour";
+static NSString* backgroundColour   = @"BackgroundColour";
+static NSString* showBorders		= @"ShowBorders";
+static NSString* showGlkBorders		= @"ShowGlkBorders";
+static NSString* showCoverPicture   = @"ShowCoverPicture";
+
 // == Global preferences ==
 
 static ZoomPreferences* globalPreferences = nil;
@@ -106,10 +117,14 @@ static NSArray* DefaultFonts(void) {
 	
 	NSMutableArray* defaultFonts = [[NSMutableArray alloc] init];
 	
-	NSFont* variableFont = [NSFont fontWithName: defaultFontName
-										   size: 12];
-	NSFont* fixedFont = [NSFont fontWithName: fixedFontName
-										size: 12];
+	NSFont* variableFont = [mgr fontWithFamily: defaultFontName
+										traits: NSUnboldFontMask
+										weight: 5
+										  size: 12];
+	NSFont* fixedFont = [mgr fontWithFamily: fixedFontName
+									 traits: NSUnboldFontMask
+									 weight: 5
+									   size: 12];
 	
 	if (variableFont == nil) variableFont = [NSFont systemFontOfSize: 12];
 	if (fixedFont == nil) fixedFont = [NSFont userFixedPitchFontOfSize: 12];
@@ -177,6 +192,15 @@ static NSArray* DefaultColours(void) {
 		[prefs setObject: DefaultColours()
 				  forKey: colours];
 		
+		[prefs setObject: [NSNumber numberWithInt: 0]
+				  forKey: foregroundColour];
+		[prefs setObject: [NSNumber numberWithInt: 7]
+				  forKey: backgroundColour];
+		[prefs setObject: [NSNumber numberWithBool: YES]
+				  forKey: showBorders];
+		[prefs setObject: [NSNumber numberWithBool: YES]
+				  forKey: showGlkBorders];
+		
 		[pool release];
 	}
 	
@@ -386,6 +410,71 @@ static NSArray* DefaultColours(void) {
 	return res;
 }
 
+- (float) textMargin {
+	[prefLock lock];
+	NSNumber* result = (NSNumber*)[prefs objectForKey: textMargin];
+	if (result == nil) result = [NSNumber numberWithFloat: 10.0];
+	[prefLock unlock];
+	
+	return [result floatValue];
+}
+
+- (BOOL) useScreenFonts {
+	[prefLock lock];
+	NSNumber* num = [prefs objectForKey: useScreenFonts];
+	BOOL result;
+	
+	if (num)
+		result = [num boolValue];
+	else
+		result = YES;
+	[prefLock unlock];
+	
+	return result;
+}
+
+- (BOOL) useHyphenation {
+	[prefLock lock];
+	NSNumber* num = [prefs objectForKey: useHyphenation];
+	BOOL result;
+	
+	if (num)
+		result = [num boolValue];
+	else
+		result = YES;
+	[prefLock unlock];
+	
+	return result;	
+}
+
+- (BOOL) useKerning {
+	[prefLock lock];
+	NSNumber* num = [prefs objectForKey: useKerning];
+	BOOL result;
+	
+	if (num)
+		result = [num boolValue];
+	else
+		result = YES;
+	[prefLock unlock];
+	
+	return result;	
+}
+
+- (BOOL) useLigatures {
+	[prefLock lock];
+	NSNumber* num = [prefs objectForKey: useLigatures];
+	BOOL result;
+	
+	if (num)
+		result = [num boolValue];
+	else
+		result = YES;
+	[prefLock unlock];
+	
+	return result;		
+}
+
 - (BOOL) keepGamesOrganised {
 	[prefLock lock];
 	BOOL result = [[prefs objectForKey: keepGamesOrganised] boolValue];
@@ -488,6 +577,36 @@ static NSArray* DefaultColours(void) {
 	[self preferencesHaveChanged];
 }
 
+- (void) setTextMargin: (float) value {
+	[prefs setObject: [NSNumber numberWithFloat: value]
+			  forKey: textMargin];
+	[self preferencesHaveChanged];
+}
+
+- (void) setUseScreenFonts: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: useScreenFonts];
+	[self preferencesHaveChanged];
+}
+
+- (void) setUseHyphenation: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: useHyphenation];
+	[self preferencesHaveChanged];
+}
+
+- (void) setUseKerning: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: useKerning];
+	[self preferencesHaveChanged];	
+}
+
+- (void) setUseLigatures: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: useLigatures];
+	[self preferencesHaveChanged];	
+}
+
 - (void) setFontRange: (NSRange) fontRange
 			 toFamily: (NSString*) newFontFamily {
 	// Sets a given range of fonts to the given family
@@ -567,7 +686,70 @@ static NSArray* DefaultColours(void) {
 	[self setFonts: newFonts];
 }
 
+// = Display preferences =
+
+- (int) foregroundColour {
+	NSNumber* val = [prefs objectForKey: foregroundColour];
+	if (val == nil) return 0;
+	return [val intValue];
+}
+
+- (int) backgroundColour {
+	NSNumber* val = [prefs objectForKey: backgroundColour];
+	if (val == nil) return 7;
+	return [val intValue];	
+}
+
+- (BOOL) showCoverPicture {
+	NSNumber* val = [prefs objectForKey: showCoverPicture];
+	if (val == nil) return YES;
+	return [val boolValue];	
+}
+
+- (BOOL) showBorders {
+	NSNumber* val = [prefs objectForKey: showBorders];
+	if (val == nil) return YES;
+	return [val boolValue];
+}
+
+- (BOOL) showGlkBorders {
+	NSNumber* val = [prefs objectForKey: showGlkBorders];
+	if (val == nil) return YES;
+	return [val boolValue];	
+}
+
+- (void) setShowCoverPicture: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: showCoverPicture];
+	[self preferencesHaveChanged];	
+}
+
+- (void) setShowBorders: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: showBorders];
+	[self preferencesHaveChanged];	
+}
+
+- (void) setShowGlkBorders: (BOOL) value {
+	[prefs setObject: [NSNumber numberWithBool: value]
+			  forKey: showGlkBorders];
+	[self preferencesHaveChanged];	
+}
+
+- (void) setForegroundColour: (int) value {
+	[prefs setObject: [NSNumber numberWithInt: value]
+			  forKey: foregroundColour];
+	[self preferencesHaveChanged];		
+}
+
+- (void) setBackgroundColour: (int) value {
+	[prefs setObject: [NSNumber numberWithInt: value]
+			  forKey: backgroundColour];
+	[self preferencesHaveChanged];	
+}
+
 // = Notifications =
+
 - (void) preferencesHaveChanged {
 	[[NSNotificationCenter defaultCenter] postNotificationName: ZoomPreferencesHaveChangedNotification
 														object:self];
