@@ -88,7 +88,7 @@
 }
 
 - (NSString*) description {
-    return @"Zoom 1.0.4 ZMachine object";
+    return @"Zoom 1.1.2 ZMachine object";
 }
 
 - (void) connectionDied: (NSNotification*) notification {
@@ -129,6 +129,8 @@
 	rc_defgame->xsize = 80;
 	rc_defgame->ysize = 25;
 	rc_defgame->antialias = 1;
+	rc_defgame->fg_col = [display foregroundColour];
+	rc_defgame->bg_col = [display backgroundColour];
 	
 	hash_store(rc_hash, "default", 7, rc_defgame);
 	
@@ -144,19 +146,19 @@
 // = Running =
 - (void) startRunningInDisplay: (in byref NSObject<ZDisplay>*) disp {
     NSAutoreleasePool* mainPool = [[NSAutoreleasePool alloc] init];
-    /*
-    {
-        int x;
-        for (x=0; x<10; x++) {
-            NSLog(@"...%i...", x);
-            sleep(1);
-        }
-    }
-     */
     
+	// Remember the display
     display = [disp retain];
+	
+	// Set up colours
+	if (rc_defgame) {
+		rc_defgame->fg_col = [display foregroundColour];
+		rc_defgame->bg_col = [display backgroundColour];
+	}
 
     // OK, we can now set up the ZMachine and get running
+	rc_defgame->interpreter = [display interpreterVersion];
+	rc_defgame->revision	= [display interpreterRevision];
 
     // Setup the display
     windows[0] = NULL;
@@ -193,7 +195,13 @@
     // Setup the display, etc
     rc_set_game(zmachine_get_serial(), Word(ZH_release), Word(ZH_checksum));
     display_initialise();
-	
+
+	// Clear the display to the default colours
+	if (!wasRestored) {
+		display_set_colour(rc_get_foreground(), rc_get_background());
+		display_clear();
+	}
+
 	if (wasRestored) zmachine_setup_header();
 
     // Start running the machine
@@ -203,7 +211,7 @@
         case 3:
             display_split(1, 1);
 
-            display_set_colour(0, 7); display_set_font(0);
+            display_set_colour(rc_get_foreground(), rc_get_background()); display_set_font(0);
             display_set_window(0);
             if (!wasRestored) zmachine_run(3, NULL); else zmachine_runsome(3, machine.zpc);
             break;
@@ -238,6 +246,9 @@
             break;
     }
 
+	stream_flush_buffer();
+	display_flush();
+
     display_finalise();
     [mainPool release];
 	
@@ -783,6 +794,9 @@ void zmachine_fatal(char* format, ...) {
 	
 	fatalBuf[511] = 0;
 	
+	stream_flush_buffer();
+	display_flush();
+	
 	[[mainMachine display] displayFatalError: [NSString stringWithFormat: @"%s (PC=#%x)", fatalBuf, machine.zpc]];
 	
 	display_exit(1);
@@ -802,5 +816,8 @@ void zmachine_warning(char* format, ...) {
 	NSLog(@"Warning: %s", fatalBuf);
 #endif
 	
+	stream_flush_buffer();
+	display_flush();
+	
 	[[mainMachine display] displayWarning: [NSString stringWithFormat: @"%s (PC=#%x)", fatalBuf, machine.zpc]];
 }
